@@ -419,7 +419,11 @@ impl<'a, A, D> NdProducer for ArrayViewMut<'a, A, D>
 /// let c = M::zeros((64, 64));
 /// let d = M::zeros((64, 64));
 ///
-/// Zip::from(&mut a).and(&b).and(&c).and(&d).apply(|w, &x, &y, &z| {
+/// Zip::from(&mut a)
+///     .and(&b)
+///     .and(&c)
+///     .and(&d)
+///     .apply(|w, &x, &y, &z| {
 ///     *w += x + y * z;
 /// });
 /// ```
@@ -434,14 +438,14 @@ impl<P, D> Zip<(P, ), D>
     where D: Dimension,
           P: NdProducer<Dim=D>
 {
-    /// Create a new `Zip` from the input array `array`.
+    /// Create a new `Zip` from the input array or other producer `p`.
     ///
-    /// The Zip will take the exact dimension of `array` and all inputs
+    /// The Zip will take the exact dimension of `p` and all inputs
     /// must have the same dimensions (or be broadcast to them).
-    pub fn from<Part>(array: Part) -> Self
-        where Part: IntoNdProducer<Dim=D, Output=P>
+    pub fn from<IP>(p: IP) -> Self
+        where IP: IntoNdProducer<Dim=D, Output=P>
     {
-        let array = array.into_producer();
+        let array = p.into_producer();
         let dim = array.raw_dim();
         Zip {
             dimension: dim,
@@ -458,8 +462,9 @@ impl<Parts, D> Zip<Parts, D>
     fn check<P>(&self, part: &P)
         where P: NdProducer<Dim=D>
     {
-        debug_assert_eq!(&self.dimension, &part.raw_dim());
-        assert!(part.equal_dim(&self.dimension));
+        ndassert!(part.equal_dim(&self.dimension),
+            "Zip: Producer dimension mismatch, expected: {:?}, got: {:?}",
+            self.dimension, part.raw_dim());
     }
 
     /// Return a the number of element tuples in the Zip
@@ -675,13 +680,8 @@ zipt_impl!{
     [A B C D E F][ a b c d e f],
 }
 
-macro_rules! macroif {
-    (true $($x:tt)*) => { $($x)* };
-    (false $($x:tt)*) => {  };
-}
-
 macro_rules! map_impl {
-    ($([$choice:ident $($p:ident)*],)+) => {
+    ($([$notlast:ident $($p:ident)*],)+) => {
         $(
         #[allow(non_snake_case)]
         impl<D: Dimension, $($p: NdProducer<Dim=D>),*> Zip<($($p,)*), D> {
@@ -711,11 +711,11 @@ macro_rules! map_impl {
                 })
             }
 
-            macroif!{ $choice
+            expand_if!(@bool [$notlast]
 
             /// Include the producer `p` in the Zip.
             ///
-            /// ***Panics*** if `p`’s shape doen't match the Zip’s exactly.
+            /// ***Panics*** if `p`’s shape doesn’t match the Zip’s exactly.
             pub fn and<P>(self, p: P) -> Zip<($($p,)* P::Output, ), D>
                 where P: IntoNdProducer<Dim=D>,
             {
@@ -749,8 +749,7 @@ macro_rules! map_impl {
                     dimension: self.dimension,
                 }
             }
-
-            }
+            );
 
             /// Split the `Zip` evenly in two.
             ///
