@@ -2,9 +2,10 @@
 extern crate num_traits;
 
 use ndarray::prelude::*;
-use ndarray::{arr0, rcarr1, rcarr2};
+use ndarray::{rcarr1, rcarr2};
 use ndarray::{LinalgScalar, Data};
 use ndarray::linalg::general_mat_mul;
+use ndarray::Si;
 
 use std::fmt;
 use num_traits::Float;
@@ -85,7 +86,7 @@ fn scalar_operations()
         let mut x = a.clone();
         let mut y = arr0(0.);
         x += 1.;
-        y.assign_scalar(&2.);
+        y.fill(2.);
         assert_eq!(x, a + arr0(1.));
         assert_eq!(x, y);
     }
@@ -94,7 +95,7 @@ fn scalar_operations()
         let mut x = b.clone();
         let mut y = rcarr1(&[0., 0.]);
         x += 1.;
-        y.assign_scalar(&2.);
+        y.fill(2.);
         assert_eq!(x, b + arr0(1.));
         assert_eq!(x, y);
     }
@@ -103,7 +104,7 @@ fn scalar_operations()
         let mut x = c.clone();
         let mut y = RcArray::zeros((2, 2));
         x += 1.;
-        y.assign_scalar(&2.);
+        y.fill(2.);
         assert_eq!(x, c + arr0(1.));
         assert_eq!(x, y);
     }
@@ -263,21 +264,21 @@ fn fold_and_sum() {
     }
 }
 
-fn range_mat(m: Ix, n: Ix) -> Array<f32, (Ix, Ix)> {
+fn range_mat(m: Ix, n: Ix) -> Array2<f32> {
     Array::linspace(0., (m * n - 1) as f32, m * n).into_shape((m, n)).unwrap()
 }
 
-fn range_mat64(m: Ix, n: Ix) -> Array<f64, (Ix, Ix)> {
+fn range_mat64(m: Ix, n: Ix) -> Array2<f64> {
     Array::linspace(0., (m * n - 1) as f64, m * n).into_shape((m, n)).unwrap()
 }
 
-fn range_i32(m: Ix, n: Ix) -> Array<i32, (Ix, Ix)> {
+fn range_i32(m: Ix, n: Ix) -> Array2<i32> {
     Array::from_iter(0..(m * n) as i32).into_shape((m, n)).unwrap()
 }
 
 // simple, slow, correct (hopefully) mat mul
-fn reference_mat_mul<A, S, S2>(lhs: &ArrayBase<S, (Ix, Ix)>, rhs: &ArrayBase<S2, (Ix, Ix)>)
-    -> Array<A, (Ix, Ix)>
+fn reference_mat_mul<A, S, S2>(lhs: &ArrayBase<S, Ix2>, rhs: &ArrayBase<S2, Ix2>)
+    -> Array2<A>
     where A: LinalgScalar,
           S: Data<Elem=A>,
           S2: Data<Elem=A>,
@@ -459,6 +460,88 @@ fn scaled_add() {
 
 }
 
+#[test]
+fn scaled_add_2() {
+    let beta = -2.3;
+    let sizes = vec![(4, 4, 1, 4),
+                     (8, 8, 1, 8),
+                     (17, 15, 17, 15),
+                     (4, 17, 4, 17),
+                     (17, 3, 1, 3),
+                     (19, 18, 19, 18),
+                     (16, 17, 16, 17),
+                     (15, 16, 15, 16),
+                     (67, 63, 1, 63),
+        ];
+    // test different strides
+    for &s1 in &[1, 2, -1, -2] {
+        for &s2 in &[1, 2, -1, -2] {
+            for &(m, k, n, q) in &sizes {
+                let mut a = range_mat64(m, k);
+                let mut answer = a.clone();
+                let c = range_mat64(n, q);
+
+                {
+                    let mut av = a.slice_mut(s![..;s1, ..;s2]);
+                    let c = c.slice(s![..;s1, ..;s2]);
+
+                    let mut answerv = answer.slice_mut(s![..;s1, ..;s2]);
+                    answerv += &(beta * &c);
+                    av.scaled_add(beta, &c);
+                }
+                assert_close(a.view(), answer.view());
+            }
+        }
+    }
+}
+
+#[test]
+fn scaled_add_3() {
+    let beta = -2.3;
+    let sizes = vec![(4, 4, 1, 4),
+                     (8, 8, 1, 8),
+                     (17, 15, 17, 15),
+                     (4, 17, 4, 17),
+                     (17, 3, 1, 3),
+                     (19, 18, 19, 18),
+                     (16, 17, 16, 17),
+                     (15, 16, 15, 16),
+                     (67, 63, 1, 63),
+        ];
+    // test different strides
+    for &s1 in &[1, 2, -1, -2] {
+        for &s2 in &[1, 2, -1, -2] {
+            for &(m, k, n, q) in &sizes {
+                let mut a = range_mat64(m, k);
+                let mut answer = a.clone();
+                let cdim = if n == 1 {
+                    vec![q]
+                } else {
+                    vec![n, q]
+                };
+                let cslice = if n == 1 {
+                    vec![Si(0, None, s2)]
+                } else {
+                    vec![Si(0, None, s1), Si(0, None, s2)]
+                };
+
+                let c = range_mat64(n, q).into_shape(cdim).unwrap();
+
+                {
+                    let mut av = a.slice_mut(s![..;s1, ..;s2]);
+                    let c = c.slice(&cslice);
+
+                    let mut answerv = answer.slice_mut(s![..;s1, ..;s2]);
+                    answerv += &(beta * &c);
+                    av.scaled_add(beta, &c);
+                }
+                assert_close(a.view(), answer.view());
+            }
+        }
+    }
+}
+
+
 #[test]
 fn gen_mat_mul() {
     let alpha = -2.3;
